@@ -87,11 +87,11 @@ enum {
 #if defined(SKF_AD_VLAN_TAG_PRESENT)
 u_int
 pcap_filter_with_aux_data(const struct bpf_insn *pc, const u_char *p,
-    u_int wirelen, u_int buflen, const struct bpf_aux_data *aux_data)
+    u_int wirelen, u_int buflen, const struct pcap_bpf_aux_data *aux_data)
 #else
 u_int
 pcap_filter_with_aux_data(const struct bpf_insn *pc, const u_char *p,
-    u_int wirelen, u_int buflen, const struct bpf_aux_data *aux_data _U_)
+    u_int wirelen, u_int buflen, const struct pcap_bpf_aux_data *aux_data _U_)
 #endif
 {
 	register uint32_t A, X;
